rm -f /usr/share/magnifier/topleft.bmp
rm -f /usr/share/magnifier/topright.bmp
rm -f /usr/share/magnifier/bottomleft.bmp
rm -f /usr/share/magnifier/bottomright.bmp
rm -f /usr/share/magnifier/top.bmp
rm -f /usr/share/magnifier/left.bmp
rm -f /usr/share/magnifier/bottom.bmp
rm -f /usr/share/magnifier/right.bmp
rm -f /usr/share/magnifier/icon3.ico
rm -f /usr/share/magnifier/icon3.png
rm -f /usr/share/magnifier/cecae.bmp
rm -f /usr/share/magnifier/feusp.bmp
rm -f /usr/share/magnifier/vmg.bmp
rm -f /usr/share/magnifier/lupa.bmp
rm -f /usr/share/magnifier/usplegal.bmp

rm -f /usr/share/magnifier/README-EN.pdf
rm -f /usr/share/magnifier/README-EN.rtf
rm -f /usr/share/magnifier/README-PT.pdf
rm -f /usr/share/magnifier/README-PT.rtf
rm -f /usr/share/magnifier/README-PT.fml

rm -f /usr/share/magnifier/magnifier

rm -rf /usr/share/magnifier

rm -f /usr/bin/vmg
rm -f /usr/share/applications/vmg.desktop

#rmdir /usr/share/magnifier
