
# Cleans the base directory

rm -f *.o
rm -f *.ppu

rm -f *.pas~
rm -f *.sh~
rm -f *.bak

rm -f *~

rm -f ppas.sh
rm -f log.txt

rm -f *.compiled

rm -f magnifier

# Cleans the build directory

cd build

rm -f *~

cd ..
