#define RT_MANIFEST  24
#define CREATEPROCESS_MANIFEST_RESOURCE_ID 1
#define ISOLATIONAWARE_MANIFEST_RESOURCE_ID 2
#define ISOLATIONAWARE_NOSTATICIMPORT_MANIFEST_RESOURCE_ID 3

CREATEPROCESS_MANIFEST_RESOURCE_ID RT_MANIFEST "dynamic_mode_windows.manifest"
MAINICON ICON "dynamic_mode_windows.ico"
