// Used by magnifier.rc
//
#define IDI_ICON1                       101
#define IDI_ICON2                       115

#define IDB_TOPLEFT                     201
#define IDB_TOPRIGHT                    202
#define IDB_BOTTOMLEFT                  203
#define IDB_BOTTOMRIGHT                 204
#define IDB_TOP                         205
#define IDB_LEFT                        206
#define IDB_BOTTOM                      207
#define IDB_RIGHT                       208

#define IDB_CECAE                       301
#define IDB_FEUSP                       302
#define IDB_VMG                         303
#define IDB_LUPA                        304
#define IDB_USPLEGAL                    305
