//{{NO_DEPENDENCIES}}
// Microsoft Visual C++ generated include file.
// Used by TestApp.rc
//
#define ID_SHOW_MAGNIFIER               3
#define ID_SHOW_MAGNIFIER2              4
#define ID_HIDE_MAGNIFIER               4
#define IDM_ABOUTBOX                    0x0010
#define IDD_ABOUTBOX                    100
#define IDS_ABOUTBOX                    101
#define IDD_TESTAPP_DIALOG              102
#define IDR_MAINFRAME                   128

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NEXT_RESOURCE_VALUE        129
#define _APS_NEXT_COMMAND_VALUE         32771
#define _APS_NEXT_CONTROL_VALUE         1000
#define _APS_NEXT_SYMED_VALUE           101
#endif
#endif
